/* Copyright (C) 2024 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer] */

/// \file   CbmQaReportElement.h
/// \brief  Base class for the report element (header)
/// \author Sergei Zharko <s.zharko@gsi.de>
/// \since  21.02.2024

#pragma once

#include "CbmQaReportDefines.h"

#include <memory>
#include <string>
#include <vector>

namespace cbm::qa::report
{
  class Engine;

  /// \class Element
  /// \brief Interface for the report element
  ///
  class Element {
   public:
    /// \brief Constructor
    /// \param label  Label of section
    Element(std::string_view label) : fsLabel(label) {}

    /// \brief Default constructor
    Element() = default;

    /// \brief Destructor
    virtual ~Element() = default;

    /// \brief Gets body of the element
    /// \param engine  A concrete implementation of the Engine to get the element body
    virtual std::string GetBody(const Engine& engine) const = 0;

    /// \brief Gets label
    const std::string& GetLabel() const { return fsLabel; }

    /// \brief Sets label
    void SetLabel(std::string_view label) { fsLabel = label; }

   private:
    // TODO: Add check for multiple label definition
    std::string fsLabel = "";  ///< Label for referencing
  };

  /// \class CollapsibleElement
  /// \brief Interface to the element, which can contain daughter elements
  ///
  class CollapsibleElement : public Element {
   public:
    using Element::Element;

    /// \brief Destructor
    virtual ~CollapsibleElement() = default;

    /// \brief Adds element
    // TODO: Check for label
    virtual void Add(std::shared_ptr<Element> pElement) { fvDaughterElements.push_back(pElement); }

    /// \brief Get daughter elements
    const std::vector<std::shared_ptr<Element>> GetDaughterElements() const { return fvDaughterElements; }

   private:
    std::vector<std::shared_ptr<Element>> fvDaughterElements;  ///< Daughter elements
  };
}  // namespace cbm::qa::report
